######################################################################################################
##
## THIS FILE CONTAINS R CODE TO REPRODUCE THE RESULTS IN ROSSELL & RUBIO
##
## 1. Simulations with p=5
## 2. High-dimensional simulations (p=100, p=500) for Bayesian methods
## 3. High-dimensional simulations (p=100, p=500) for penalized likelihood methods
## 4. Summarize results of high-dimensional simulations (tables, plots)
## 5. Analysis of TGFB data
## 6. Analysis of DLD data
##
######################################################################################################

library(mombf)
library(twopiece)  #at https://r-forge.r-project.org/projects/twopiece
library(parallel)  #if not available replace mclapply below for lapply
source('routines_simulation.R')

##########################################################################################################
## 1. SIMULATION WITH P=5
##########################################################################################################

## 1.1 Normal errors ##
#######################

#priorSkew |alpha|>0.2
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=2)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)



#priorSkew |alpha|>0.1
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.087); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=2)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)


## 1.2 Skew normal errors ##
############################

#priorSkew |alpha|>0.2
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=2)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)


#priorSkew |alpha|>0.1
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.087); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=2)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)


## 1.3 Laplace errors ##
########################

#priorSkew |alpha|>0.2
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i,family='laplace',n=n,theta=theta,vartheta=vartheta,alpha=alpha,rho=rho,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta), mc.cores=6)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)


#priorSkew |alpha|>0.1
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.087); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i,family='laplace',n=n,theta=theta,vartheta=vartheta,alpha=alpha,rho=rho,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta), mc.cores=6)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)


## 1.4 Skew laplace errors ##
#############################

#priorSkew |alpha|>0.2
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i,family='laplace',n=n,theta=theta,vartheta=vartheta,alpha=alpha,rho=rho,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta), mc.cores=6)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)

#priorSkew |alpha|>0.1
set.seed(12345)
nsim <- 200; n=100; p <- 5; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,0,0)  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.087); priorDelta <- modelunifprior()
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i,family='laplace',n=n,theta=theta,vartheta=vartheta,alpha=alpha,rho=rho,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta), mc.cores=6)

#Marginal posterior probabilities
margpp.n <- do.call(rbind,lapply(sims.out,'[[','margpp.n'))
margpp.skewn <- do.call(rbind,lapply(sims.out,'[[','margpp.skewn'))
margpp.l <- do.call(rbind,lapply(sims.out,'[[','margpp.l'))
margpp.skewl <- do.call(rbind,lapply(sims.out,'[[','margpp.skewl'))
margpp.auto <- do.call(rbind,lapply(sims.out,'[[','margpp.auto'))
xtab <- rbind(colMeans(margpp.n),colMeans(margpp.skewn),colMeans(margpp.l),colMeans(margpp.skewl),colMeans(margpp.auto)[1:6])[,-1]
round(colMeans(margpp.auto[,-1:-6]),2)



##########################################################################################################
## 2. HIGH-DIMENSIONAL SIMULATION
##########################################################################################################

## 2.1 P=100 ##
###############

#Normal errors
nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_normal_p100.RData')

#Skew normal errors
nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_tpnormal_p100.RData')

# Laplace errors
nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_lapl_p100.RData')

# Skew laplace errors
nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_tplapl_p100.RData')


## 1.2 SIMULATION WITH P=500 ##
###############################

#Normal errors
nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_normal_p500.RData')

# Skew normal errors
nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_tpnormal_p500.RData')

#Laplace errors
nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- 0
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_lapl_p500.RData')

# Skew laplace errors
nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- -0.5
theta <- c(0,.5,0.75,1,rep(0,p-3))  #first elem corresponds to intercept
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
sims.out <- mclapply(1:nsim, function(i) simfit(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta), mc.cores=6)
save(sims.out,file='sims_tplapl_p500.RData')


##########################################################################################################
## 3. PENALIZED LIKELIHOOD SIMULATIONS
##########################################################################################################

library(lars); library(parcor); library(flare); library(ncvreg)

#vartheta=2
nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_normal_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_normal_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tpnormal_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tpnormal_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_lapl_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_lapl_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 2; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tplapl_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tplapl_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_normal_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_normal_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tpnormal_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tpnormal_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_lapl_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_lapl_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 2; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tplapl_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tplapl_p',p,'.RData',sep=''))


#vartheta=1
nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 1; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_normal_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_normal_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 1; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tpnormal_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tpnormal_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 1; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_lapl_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_lapl_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 100; rho <- 0.5; vartheta <- 1; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tplapl_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tplapl_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 1; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_normal_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_normal_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 1; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tpnormal_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tpnormal_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 1; alpha <- 0; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_lapl_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_lapl_vtheta1_p',p,'.RData',sep=''))

nsim <- 200; n=100; p <- 500; rho <- 0.5; vartheta <- 1; alpha <- -0.5; theta <- c(0,.5,0.75,1,rep(0,p-3))
load(paste('simspen_tplapl_vtheta1_p',p,'.RData',sep=''))
sims.pen <- mclapply(1:nsim, function(i) simpenlhood(seed=i, family='laplace', n=n, theta=theta, vartheta=vartheta, alpha=alpha, rho=rho, sims.pen=sims.pen[[i]]), mc.cores=6)
save(sims.pen,file=paste('simspen_tplapl_vtheta1_p',p,'.RData',sep=''))


##########################################################################################################
## 4. SUMMARIZE RESULTS
##########################################################################################################

## 4.1 vartheta=1 ##
####################

#Truly normal residuals, vartheta=1
load('sims_normal_vtheta1_p100.RData')
load('simspen_normal_vtheta1_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_normal_vtheta1_p500.RData')
load('simspen_normal_vtheta1_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab


col <- c('blue','blue','red','red','black',rep('darkgray',3),'black'); lty <- c(1,2,1,2,1,1,2,3,3)
plot(x2plot[,'p'], x2plot[,'Normal'], type='l', col=col[1], lty=lty[1], ylim=c(0,0.8), ylab='Proportion of correct model selections', xlab='p', xaxt='n', cex.lab=1.3, cex.axis=1.3)
lines(x2plot[,'p'], x2plot[,'Two-piece Normal'], col=col[2], lty=lty[2])
lines(x2plot[,'p'], x2plot[,'Laplace'], col=col[3], lty=lty[3])
lines(x2plot[,'p'], x2plot[,'Two-piece Laplace'], col=col[4], lty=lty[4])
lines(x2plot[,'p'], x2plot[,'Inferred'], col=col[5], lty=lty[5])
lines(x2plot[,'p'], x2plot[,'LASSO-LS'], col=col[6], lty=lty[6])
lines(x2plot[,'p'], x2plot[,'LASSO-LAD'], col=col[7], lty=lty[7])
lines(x2plot[,'p'], x2plot[,'SCAD'], col=col[9], lty=lty[9])
axis(1,at=x2plot[,'p'], cex.axis=1.3)
text(x2plot[1,'p'],x2plot[1,c(-1,-9)]+c(.03,.01,0,0,-.03,0,-.015,0),c('Normal','ANormal','Laplace','ALaplace','Inferred','LASSO-LS','LASSO-LAD/LASSO-QR','SCAD'),pos=4,cex=1.2)


#Truly tp-normal residuals, vartheta=1
load('sims_tpnormal_vtheta1_p100.RData')
load('simspen_tpnormal_vtheta1_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_tpnormal_vtheta1_p500.RData')
load('simspen_tpnormal_vtheta1_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab

col <- c('blue','blue','red','red','black',rep('darkgray',3),'black'); lty <- c(1,2,1,2,1,1,2,3,3)
plot(x2plot[,'p'], x2plot[,'Normal'], type='l', col=col[1], lty=lty[1], ylim=c(0,0.8), ylab='Proportion of correct model selections', xlab='p', xaxt='n', cex.lab=1.3, cex.axis=1.3)
lines(x2plot[,'p'], x2plot[,'Two-piece Normal'], col=col[2], lty=lty[2])
lines(x2plot[,'p'], x2plot[,'Laplace'], col=col[3], lty=lty[3])
lines(x2plot[,'p'], x2plot[,'Two-piece Laplace'], col=col[4], lty=lty[4])
lines(x2plot[,'p'], x2plot[,'Inferred'], col=col[5], lty=lty[5])
lines(x2plot[,'p'], x2plot[,'LASSO-LS'], col=col[6], lty=lty[6])
lines(x2plot[,'p'], x2plot[,'LASSO-LAD'], col=col[7], lty=lty[7])
lines(x2plot[,'p'], x2plot[,'LASSO-QR'], col=col[8], lty=lty[8])
lines(x2plot[,'p'], x2plot[,'SCAD'], col=col[9], lty=lty[9])
axis(1,at=x2plot[,'p'], cex.axis=1.3)
text(x2plot[1,'p'],x2plot[1,-1]+c(.015,0,0,0,-.01,.02,-.02,0,0),c('Normal','ANormal','Laplace','ALaplace','Inferred','LASSO-LS','LASSO-LAD','LASSO-QR','SCAD'),pos=4,cex=1.2)



#Truly Laplace residuals, vartheta=1
load('sims_lapl_vtheta1_p100.RData')
load('simspen_lapl_vtheta1_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_lapl_vtheta1_p500.RData')
load('simspen_lapl_vtheta1_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab

col <- c('blue','blue','red','red','black',rep('darkgray',3),'black'); lty <- c(1,2,1,2,1,1,2,3,3)
plot(x2plot[,'p'], x2plot[,'Normal'], type='l', col=col[1], lty=lty[1], ylim=c(0,0.8), ylab='Proportion of correct model selections', xlab='p', xaxt='n', cex.lab=1.3, cex.axis=1.3)
lines(x2plot[,'p'], x2plot[,'Two-piece Normal'], col=col[2], lty=lty[2])
lines(x2plot[,'p'], x2plot[,'Laplace'], col=col[3], lty=lty[3])
lines(x2plot[,'p'], x2plot[,'Two-piece Laplace'], col=col[4], lty=lty[4])
lines(x2plot[,'p'], x2plot[,'Inferred'], col=col[5], lty=lty[5])
lines(x2plot[,'p'], x2plot[,'LASSO-LS'], col=col[6], lty=lty[6])
lines(x2plot[,'p'], x2plot[,'LASSO-LAD'], col=col[7], lty=lty[7])
lines(x2plot[,'p'], x2plot[,'SCAD'], col=col[9], lty=lty[9])
axis(1,at=x2plot[,'p'], cex.axis=1.3)
text(x2plot[1,'p'],x2plot[1,c(-1,-9)]+c(0,.01,.02,-.01,0,-.015,0,.02),c('Normal','ANormal','Laplace','ALaplace','Inferred','LASSO-LS','LASSO-LAD/LASSO-QR','SCAD'),pos=4,cex=1.2)



#Truly Two-piece Laplace residuals, vartheta=1
load('sims_tplapl_vtheta1_p100.RData')
load('simspen_tplapl_vtheta1_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_tplapl_vtheta1_p500.RData')
load('simspen_tplapl_vtheta1_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab

col <- c('blue','blue','red','red','black',rep('darkgray',3),'black'); lty <- c(1,2,1,2,1,1,2,3,3)
plot(x2plot[,'p'], x2plot[,'Normal'], type='l', col=col[1], lty=lty[1], ylim=c(0,0.8), ylab='Proportion of correct model selections', xlab='p', xaxt='n', cex.lab=1.3, cex.axis=1.3)
lines(x2plot[,'p'], x2plot[,'Two-piece Normal'], col=col[2], lty=lty[2])
lines(x2plot[,'p'], x2plot[,'Laplace'], col=col[3], lty=lty[3])
lines(x2plot[,'p'], x2plot[,'Two-piece Laplace'], col=col[4], lty=lty[4])
lines(x2plot[,'p'], x2plot[,'Inferred'], col=col[5], lty=lty[5])
lines(x2plot[,'p'], x2plot[,'LASSO-LS'], col=col[6], lty=lty[6])
lines(x2plot[,'p'], x2plot[,'LASSO-LAD'], col=col[7], lty=lty[7])
lines(x2plot[,'p'], x2plot[,'LASSO-QR'], col=col[8], lty=lty[8])
lines(x2plot[,'p'], x2plot[,'SCAD'], col=col[9], lty=lty[9])
axis(1,at=x2plot[,'p'], cex.axis=1.3)
text(x2plot[1,'p'],x2plot[1,-1]+c(0,0,0,.01,0,-.025,-.005,.01,.02),c('Normal','ANormal','Laplace','ALaplace','Inferred','LASSO-LS','LASSO-LAD','LASSO-QR','SCAD'),pos=4,cex=1.2)


## 4.2 vartheta=2 ##
####################

#Truly normal residuals, vartheta=2
load('sims_normal_p100.RData')
load('simspen_normal_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_normal_p500.RData')
load('simspen_normal_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab


#Truly tp-normal residuals, vartheta=2
load('sims_tpnormal_p100.RData')
load('simspen_tpnormal_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_tpnormal_p500.RData')
load('simspen_tpnormal_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab




#Truly Laplace residuals, vartheta=2
load('sims_lapl_p100.RData')
load('simspen_lapl_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_lapl_p500.RData')
load('simspen_lapl_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab


#Truly Two-piece Laplace residuals, vartheta=2
load('sims_tplapl_p100.RData')
load('simspen_tplapl_p100.RData')
p <- 100; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p100 <- getCorrect(sims.out,theta=theta)
pen.p100 <- getCorrect.pen(sims.pen,theta=theta)
#
load('sims_tplapl_p500.RData')
load('simspen_tplapl_p500.RData')
p <- 500; theta <- c(0,.5,0.75,1,rep(0,p-3))
nlp.p500 <- getCorrect(sims.out,theta=theta)
pen.p500 <- getCorrect.pen(sims.pen,theta=theta)

xtab.100 <- rbind(data.frame(nlp.p100),data.frame(method=pen.p100[,1],pptruemodel.n=NA,pen.p100[,-1]))
xtab.500 <- rbind(data.frame(nlp.p500),data.frame(method=pen.p500[,1],pptruemodel.n=NA,pen.p500[,-1]))
xtab <- cbind(xtab.100[,-4],xtab.500[,c(-1,-4)]); rownames(xtab) <- NULL
xtab




##########################################################################################################
## 5. TGFB DATA (provided in Rossell & Telesca, Non-local priors for high-dimensional estimation, JASA 2016
##########################################################################################################

#Fit models
x <- read.table('tgfb_10000.txt',header=TRUE,sep='\n')
y <- as.vector(scale(x[,1])); x <- scale(as.matrix(x[,-1]))
priorCoef <- momprior(tau=.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(alpha.p=1,beta.p=1); priorVar <- igprior(alpha=.01, lambda=.01)
msn <- modelSelection(y=y, x=x, niter=1000, priorCoef=priorCoef, priorDelta=priorDelta, priorVar=priorVar, method='auto', family='normal')
msa <- modelSelection(y=y, x=x, niter=1000, priorCoef=priorCoef, priorDelta=priorDelta, priorVar=priorVar, method='auto', family='auto')


#Get results
load('geneexpr_fit_1000iter.RData')
margpp.norm <- function(ms) {
  pp <- postProb(ms,method='norm')
  models <- do.call(rbind,lapply(strsplit(as.character(pp[,1]),','), function(z) { ans <- rep(FALSE,ncol(ms$postSample)); ans[as.numeric(z)] <- TRUE; return(ans) }))
  margpp.n <- colSums(models * pp[,'pp'])
  return(margpp.n)
}


ppn <- postProb(msn,nmax=50); margppn <- margpp.norm(msn)
ppa <- postProb(msa,nmax=50); margppa <- margpp.norm(msa)

sum(msn$postMode); sum(margppn) #Model size under normal model
sum(msa$postMode[1:ncol(x)]); sum(margppa[1:ncol(x)]) #Model size under inferred residual distrib
msa$margpp[length(msa$margpp)]  #P(alpha != 0 |y)

ppn[1,]; ppa[1,] #top model under Normal errors; top model under inferred errors

#Residual QQ-plot under top model
sel <- as.numeric(strsplit(as.character(ppn[1,'modelid']),split=',')[[1]])
lm1 <- lm(y ~ ., data=data.frame(x[,sel]))
qqnorm(scale(residuals(lm1)), cex.lab=1.25, main=''); abline(0,1)

#Obtain names of the selected genes
library(hgu133plus2.db)
ids <- sub("^X","",colnames(x)[sel])
data.frame(gene=unlist(mget(ids,hgu133plus2SYMBOL)),margppn[sel])



##########################################################################################################
## 6. GSE71008 DATA (provided as supplement to Rossell and Rubio, Tractable Bayesian variable selection: beyond normality)
##########################################################################################################

x <- read.table('dld.txt',sep='\t',header=TRUE)
y <- x[,1]; x <- cbind(rep(1,nrow(x)),x[,-1])
priorCoef <- momprior(tau=0.348); priorSkew <- momprior(tau=0.357); priorDelta <- modelbbprior(1,1)
fitn <- modelSelection(y, x=x, family='twopiecenormal', priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta, niter=10^4, method='auto')
fita <- modelSelection(y, x=x, family='auto', priorCoef=priorCoef, priorSkew=priorSkew, priorDelta=priorDelta, niter=10^4, method='Laplace')

#Top models
head(postProb(fitn))
head(postProb(fita))

fita$margpp[-1:-ncol(x)] #inference on residual distribution

#Residual QQ-plot under top model
sel <- as.numeric(strsplit(as.character(postProb(fitn)[1,'modelid']), split=',')[[1]])
xsel <- as.matrix(x[,sel])
lm1 <- lm(y ~ -1 + xsel)
qqnorm(scale(residuals(lm1)),main=''); abline(0,1)

#Marginal inclusion probabilities under Normal and inferred error distribution
margpp <- data.frame(symbol=colnames(x),normal=round(fitn$margpp,3),auto=round(fita$margpp[1:ncol(x)],3))
margpp <- margpp[order(margpp$auto,decreasing=TRUE),]
head(margpp,n=8)



#Quantile regression (corresponds to fixed asymmetry parameter alpha)
priorCoef <- momprior(tau=0.348); priorDelta <- modelbbprior(1,1)
fitalpha0 <- modelSelection(y, x=x, family='twopiecelaplace', priorCoef=priorCoef, priorSkew=0, priorDelta=priorDelta, niter=10^4) #median regression
fitalpha1 <- modelSelection(y, x=x, family='twopiecelaplace', priorCoef=priorCoef, priorSkew=atanh(-0.5), priorDelta=priorDelta, niter=10^4) #first quartile
fitalpha2 <- modelSelection(y, x=x, family='twopiecelaplace', priorCoef=priorCoef, priorSkew=atanh(0.5), priorDelta=priorDelta, niter=10^4) #third quartile

tab1 <- postProb(fitalpha0)[1:5,c(1,3)]
tab2 <- postProb(fitalpha1)[1:5,c(1,3)]
tab3 <- postProb(fitalpha2)[1:5,c(1,3)]
tab1[,1] <- sapply(strsplit(as.character(tab1[,1]),split=','), function(z) paste(names(x)[as.numeric(z)],collapse=', '))
tab2[,1] <- sapply(strsplit(as.character(tab2[,1]),split=','), function(z) paste(names(x)[as.numeric(z)],collapse=', '))
tab3[,1] <- sapply(strsplit(as.character(tab3[,1]),split=','), function(z) paste(names(x)[as.numeric(z)],collapse=', '))

xtable(tab1,digits=c(0,0,3)) #alpha= 0
xtable(tab2,digits=c(0,0,3)) #alpha= -0.5
xtable(tab3,digits=c(0,0,3)) #alpha= 0.5

#Marginal inclusion probabilities
# - Genes C6orf226, ECH1, CSF2RA, FBXL19, RRP1B appear in the top model for all alpha's (all have margpp>0.99)
# - Gene MTMR1 features only for alpha=0
common <- which(names(x) %in% c('C6orf226','ECH1','CSF2RA','FBXL19','RRP1B'))
fitalpha0$margpp[common]
fitalpha1$margpp[common]
fitalpha2$margpp[common]

spec <- which(names(x) %in% c('MTMR1'))
round(rbind(fitalpha1$margpp[spec],fitalpha0$margpp[spec],fitalpha2$margpp[spec]),3)

