######################################################################################################
##
## THIS FILE CONTAINS ALL AUXILIARY ROUTINES NEEDED TO PRODUCE THE SIMULATION STUDY IN ROSSELL & RUBIO
##
## 1. Figures
## 2. Simulate data and apply Bayesian or penalized likelihood methods
## 3. Summarize simulation results
##
######################################################################################################

##########################################################################################
##  1. ROUTINES TO PRODUCE FIGURES
###########################################################################################

#Function myvioplot is an adapted version of vioplot from package vioplot
myvioplot <- function (x, ..., range = 1.5, h = NULL, ylim = NULL, names = NULL, horizontal = FALSE, col = "magenta", border = "black", lty = 1, lwd = 1, rectCol = "black", colMed = "white", pchMed = 19,
    at, add = FALSE, wex = 1, drawRect = TRUE, plotmean=TRUE, cexMed=1, cex.axis=1, xaxt='s', yaxt='s') {
    require(vioplot)
    require(sm)
    datas <- list(x, ...)
    n <- length(datas)
    if (missing(at)) at <- 1:n
    upper <- vector(mode = "numeric", length = n)
    lower <- vector(mode = "numeric", length = n)
    q1 <- vector(mode = "numeric", length = n)
    q3 <- vector(mode = "numeric", length = n)
    med <- vector(mode = "numeric", length = n)
    avg <- vector(mode = "numeric", length = n)
    base <- vector(mode = "list", length = n)
    height <- vector(mode = "list", length = n)
    baserange <- c(Inf, -Inf)
    args <- list(display = "none")
    if (!(is.null(h))) args <- c(args, h = h)
    for (i in 1:n) {
        data <- datas[[i]]
        data.min <- min(data, na.rm=TRUE)
        data.max <- max(data, na.rm=TRUE)
        q1[i] <- quantile(data, 0.25, na.rm=TRUE)
        q3[i] <- quantile(data, 0.75, na.rm=TRUE)
        med[i] <- median(data, na.rm=TRUE)
        avg[i] <- mean(data, na.rm=TRUE)
        iqd <- q3[i] - q1[i]
        upper[i] <- min(q3[i] + range * iqd, data.max)
        lower[i] <- max(q1[i] - range * iqd, data.min)
        est.xlim <- c(min(lower[i], data.min), max(upper[i],data.max))
        smout <- do.call("sm.density", c(list(data, xlim = est.xlim),args))
        hscale <- 0.4/max(smout$estimate) * wex
        base[[i]] <- smout$eval.points
        height[[i]] <- smout$estimate * hscale
        t <- range(base[[i]])
        baserange[1] <- min(baserange[1], t[1])
        baserange[2] <- max(baserange[2], t[2])
    }
    if (!add) {
        xlim <- if (n == 1) at + c(-0.5, 0.5)
        else range(at) + min(diff(at))/2 * c(-1, 1)
        if (is.null(ylim)) { ylim <- baserange }
    }
    if (is.null(names)) { label <- 1:n } else { label <- names }
    boxwidth <- 0.05 * wex
    if (!add) plot.new()
    if (!horizontal) {
        if (!add) {
            plot.window(xlim = xlim, ylim = ylim)
            if (yaxt != 'n') axis(2, cex.axis=cex.axis)
            if (xaxt != 'n') axis(1, at = at, label = label, cex.axis=cex.axis)
        }
        box()
        for (i in 1:n) {
            polygon(c(at[i] - height[[i]], rev(at[i] + height[[i]])), c(base[[i]], rev(base[[i]])), col = col[i], border = border, lty = lty, lwd = lwd)
            if (drawRect) {
                lines(at[c(i, i)], c(lower[i], upper[i]), lwd = lwd, lty = lty)
                rect(at[i] - boxwidth/2, q1[i], at[i] + boxwidth/2, q3[i], col = rectCol)
                if (plotmean) {
                    points(at[i], avg[i], pch = pchMed, col = colMed, cex= cexMed)
                } else {
                    points(at[i], med[i], pch = pchMed, col = colMed, cex= cexMed)
                }
            }
        }
    }
    else {
        if (!add) {
            plot.window(xlim = ylim, ylim = xlim)
            axis(1)
            axis(2, at = at, label = label)
        }
        box()
        for (i in 1:n) {
            polygon(c(base[[i]], rev(base[[i]])), c(at[i] - height[[i]], rev(at[i] + height[[i]])), col = col, border = border, lty = lty, lwd = lwd)
            if (drawRect) {
                lines(c(lower[i], upper[i]), at[c(i, i)], lwd = lwd, lty = lty)
                rect(q1[i], at[i] - boxwidth/2, q3[i], at[i] + boxwidth/2, col = rectCol)
                points(med[i], at[i], pch = pchMed, col = colMed)
            }
        }
    }
    invisible(list(upper = upper, lower = lower, median = med,
        q1 = q1, q3 = q3))
}


###########################################################################################
## 2. ROUTINES TO SIMULATE DATA
###########################################################################################

simfit <- function(seed,n,theta,vartheta,alpha,family='normal',rho,enumerate=ifelse(length(theta)<=21,TRUE,FALSE),priorCoef,priorSkew,priorVar=igprior(alpha=.01,lambda=.01),priorDelta=modelbbprior(alpha.p=1,beta.p=1),niter=10^4,hess='asymp',heterosk=FALSE,heterosym=FALSE,...) {
    #Simulate linear regression data with two-piece normal (if family=='normal') or two-piece laplace (family=='laplace') errors and obtain posterior model probabilities. alpha=0 corresponds to symmetric normal/laplace.
    # - seed: random number generation seed
    # - n: sample size
    # - theta,vartheta,alpha: true parameter values  (note: first element in theta corresponds to intercept)
    # - rho: pairwise correlation between covariates
    # - enumerate: set to TRUE for full enumeration of all models, else MCMC on model space is run
    # - priorCoef: prior on theta
    # - priorSkew: prior on skewness coefficient alpha
    # - priorVar: prior on residual scale parameter
    # - priorDelta: prior on model space
    # - niter: number of iterations in MCMC model space search (only used if enumerate==FALSE)
    # - heterosk: if TRUE variance is proportional to exp(mu) where mu is the mean
    # - heterosym: if TRUE asymmetry is tanh(atanh(alpha) + rnorm(0,sd=0.25))
    # - ...: other arguments to be passed on to modelSelection
    # Output
    # - pp.n: posterior model probabilities assuming normal residuals
    # - margpp.n: marginal inclusion probabilities assuming normal residuals
    # - pp.skewn: posterior model probabilities assuming two-piece normal residuals
    # - margpp.skewn: marginal inclusion probabilities assuming two-piece normal residuals
  set.seed(seed)
  p <- length(theta)-1
  S <- diag(p); S[upper.tri(S)] <- rho; S[lower.tri(S)] <- rho
  #Simulate data
  X = cbind(rep(1,n),rmvnorm(n=n, sigma=S))
  mu <- X %*% matrix(theta,ncol=1)
  if (family=='normal') {
      if (!heterosym) {
        e= rtp3(n,0,par1=vartheta,par2=alpha,FUN=rnorm,param="eps")
      } else {
        sd0= sd(rtp3(n,0,par1=vartheta,par2=alpha,FUN=rnorm,param="eps"))
        alpha= tanh(rnorm(n,atanh(alpha),sd=.25))
        e= sapply(alpha, function(z) rtp3(1,0,par1=vartheta,par2=z,FUN=rnorm,param="eps"))
        e= e * sd0 / sd(e)
      }
  } else if (family=='laplace') {
      if (!heterosym) {
        e=  ralapl(n=n, th=0, scale=vartheta, alpha=alpha)
      } else {
        sd0=  sd(ralapl(n=n, th=0, scale=vartheta, alpha=alpha))
        alpha= tanh(rnorm(n,atanh(alpha),sd=.25))
        e=  sapply(alpha, function(z) ralapl(1,th=0,scale=vartheta,alpha=z))
        e= e * sd0 / sd(e)
      }
  } else { stop("Invalid value for argument 'family'") }
  if (heterosk) {
    enew <- e * sqrt(exp(mu))
    e <- enew * sd(e) / sd(enew)
  }
  y = mu + e
  #Assume Normal residuals
  fit.n <- modelSelection(y=y,x=X,family='normal',center=FALSE,scale=FALSE,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta,enumerate=enumerate,verbose=FALSE,...)
  pp.n <- postProb(fit.n)
  margpp.n <- fit.n$margpp
  #Assume two-piece Normal residuals
  fit.skewn <- modelSelection(y=y,x=X,family='twopiecenormal',center=FALSE,scale=FALSE,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta,enumerate=enumerate,verbose=FALSE,...)
  pp.skewn <- postProb(fit.skewn)
  margpp.skewn <- fit.skewn$margpp
  #Assume Laplace residuals
  fit.l <- modelSelection(y=y,x=X,family='laplace',center=FALSE,scale=FALSE,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta,enumerate=enumerate,verbose=FALSE,...)
  pp.l <- postProb(fit.l)
  margpp.l <- fit.l$margpp
  #Assume two-piece Laplace residuals
  fit.skewl <- modelSelection(y=y,x=X,family='twopiecelaplace',center=FALSE,scale=FALSE,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta,enumerate=enumerate,verbose=FALSE,...)
  pp.skewl <- postProb(fit.skewl)
  margpp.skewl <- fit.skewl$margpp
  #Infer residual distribution
  fit.auto <- modelSelection(y=y,x=X,family='auto',center=FALSE,scale=FALSE,priorCoef=priorCoef,priorSkew=priorSkew,priorDelta=priorDelta,enumerate=enumerate,verbose=FALSE,...)
  pp.auto <- postProb(fit.auto)
  margpp.auto <- fit.auto$margpp
  cat(seed,'\n')
  return(list(pp.n=pp.n, margpp.n=margpp.n, pp.skewn=pp.skewn, margpp.skewn=margpp.skewn, pp.l=pp.l, margpp.l=margpp.l, pp.skewl=pp.skewl, margpp.skewl=margpp.skewl, pp.auto=pp.auto, margpp.auto=margpp.auto))
}


simpenlhood <- function(seed,n,theta,vartheta,alpha,family='normal',rho,heterosk=FALSE,heterosym=FALSE,sims.pen) {
    #Simulate linear regression data with two-piece normal (if family=='normal') or two-piece laplace (family=='laplace') errors and perform variable selection via penalized likelihood
    # - seed: random number generation seed
    # - n: sample size
    # - theta,vartheta,alpha: true parameter values  (note: first element in theta corresponds to intercept)
    # - rho: pairwise correlation between covariates
    # - heterosk: if TRUE variance is proportional to exp(mu) where mu is the mean
    # - heterosym: if TRUE asymmetry is tanh(atanh(alpha) + rnorm(0,sd=0.25))
    # - sims.pen: output from earlier call to simpenlhood, useful when adding new methods so that previous simulations do not need to be run again
    # Output
    # - lasso.ls: binary vector indicating selected variables by LASSO-Least squares
    # - lasso.lad: binary vector indicating selected variables by LASSO-Least absolute deviation (uses R package flare)
    # - lasso.qr: binary vector indicating selected variables by LASSO-Quantile regression at the true quantile (uses R package rqPen)
    # - lasso.qr0: same as lasso.qr forcing the 0.5 quantile. This is equivalent to LASSO-LAD but uses the rqPen package
  set.seed(seed)
  p <- length(theta)-1
  S <- diag(p); S[upper.tri(S)] <- rho; S[lower.tri(S)] <- rho
  #Simulate data
  X = cbind(rep(1,n),rmvnorm(n=n, sigma=S))
  mu <- X %*% matrix(theta,ncol=1)
  if (family=='normal') {
      if (!heterosym) {
        e= rtp3(n,0,par1=vartheta,par2=alpha,FUN=rnorm,param="eps")
      } else {
        sd0= sd(rtp3(n,0,par1=vartheta,par2=alpha,FUN=rnorm,param="eps"))
        alpha= tanh(rnorm(n,atanh(alpha),sd=.25))
        e= sapply(alpha, function(z) rtp3(1,0,par1=vartheta,par2=z,FUN=rnorm,param="eps"))
        e= e * sd0 / sd(e)
      }
  } else if (family=='laplace') {
      if (!heterosym) {
        e=  ralapl(n=n, th=0, scale=vartheta, alpha=alpha)
      } else {
        sd0=  sd(ralapl(n=n, th=0, scale=vartheta, alpha=alpha))
        alpha= tanh(rnorm(n,atanh(alpha),sd=.25))
        e=  sapply(alpha, function(z) ralapl(1,th=0,scale=vartheta,alpha=z))
        e= e * sd0 / sd(e)
      }
  } else { stop("Invalid value for argument 'family'") }
  if (heterosk) {
    enew <- e * sqrt(exp(mu))
    e <- enew * sd(e) / sd(enew)
  }
  y = mu + e
  if (!missing(sims.pen)) { nn = names(sims.pen$coef) } else { nn <- character(0) }
  #LASSO-QR
  if (!('lasso.qr.coef' %in% nn)) {
    require(rqPen)
    tlasso.qr <- system.time(lasso.qr.coef <- lassoqrfit(y=y,x=X,alpha=alpha))['elapsed']
  } else { lasso.qr.coef <- sims.pen$coef$lasso.qr.coef; tlasso.qr <- sims.pen$time['lasso.qr.elapsed'] }
  #LASSO-QR0
  if (!('lasso.qr0.coef' %in% nn)) {
    require(rqPen)
    tlasso.qr0 <- system.time(lasso.qr0.coef <- lassoqrfit(y=y,x=X,alpha=0))['elapsed']
  } else { lasso.qr0.coef <- sims.pen$coef$lasso.qr0.coef; tlasso.qr0 <- sims.pen$time['lasso.qr0.elapsed'] }
  #LASSO-LSE
  if (!('lasso.ls.coef' %in% nn)) {
      require(lars)
      require(parcor)
      tlasso.ls <- system.time(lasso.ls.coef <- larscvfit(y=y,x=X))['elapsed']
  } else { lasso.ls.coef <- sims.pen$coef$lasso.ls.coef; tlasso.ls <- sims.pen$time['lasso.ls.elapsed'] }
  if (!('lasso.lad.coef' %in% nn)) {
      require(flare)
      tlasso.lad <- system.time(lasso.lad.coef <- lassoladcvfit(y=y,x=X))['elapsed']
  } else { lasso.lad.coef <- sims.pen$coef$lasso.lad.coef; tlasso.lad <- sims.pen$time['lasso.lad.elapsed'] }
  if (!('scad.coef' %in% nn)) {
      require(ncvreg)
      tscad <- system.time(scad.coef <- scadcvfit(y=y,x=X))['elapsed']
  } else { scad.coef <- sims.pen$coef$scad.coef; tscad <- sims.pen$time['scad.elapsed'] }
  coef= data.frame(lasso.ls.coef=lasso.ls.coef,lasso.lad.coef=lasso.lad.coef,scad.coef=scad.coef,lasso.qr.coef=lasso.qr.coef,lasso.qr0.coef=lasso.qr0.coef)
  cat(seed,'\n')
  return(list(coef=coef,time=c(lasso.ls=tlasso.ls,lasso.lad=tlasso.lad,tscad=tscad,lasso.qr=tlasso.qr,lasso.qr0=tlasso.qr0)))
}

larscvfit <- function(y,x) {
  #Auxiliary function fitting LASSO via 10-fold cross-validation
  mylars(X=x,y=y,k=10,use.Gram=TRUE,normalize=TRUE)$coef
}

scadcvfit <- function(y,x) {
  #Auxiliary function fitting SCAD via 10-fold cross-validation
  cvscad <- cv.ncvreg(X=x,y=y,family="gaussian",penalty="SCAD",nfolds=10,dfmax=1000,max.iter=10^4)
  ncvreg(X=x,y=y,penalty='SCAD',dfmax=1000,lambda=cvscad$lambda.min)$beta[-1,1]
}

lassoqrfit <- function(y,x,alpha,k=10) {
  #Remove intercept
  n <- length(y)
  y <- y-mean(y)
  x <- x[,apply(x,2,'sd')>0] #remove intercept
  x <- scale(x,center=TRUE,scale=FALSE)
  #Define lambda values
  lambda.max = max(abs(crossprod(x, sign(y)/n)))
  lambda.min.value = sqrt(log(ncol(x))/n)/4
  lambda = exp(seq(log(lambda.max), log(lambda.min.value), length=10))
  #Run cross-validation
  all.folds <- split(sample(1:n), rep(1:k, length = n))
  ypred <- matrix(nrow=n,ncol=k)
  fit = matrix(0,ncol=dim(x)[2]+1,nrow=length(lambda))
  for (i in seq(k)) {
    omit <- all.folds[[i]]
    for(l in 1:length(lambda)){
      fit[l,] <- rq.lasso.fit(x[-omit,,drop=FALSE],y[-omit], method="lasso", lambda=lambda[l], tau = 0.5*(1+alpha))$coefficients
    }
    ypred[omit,] <- x[omit,] %*% t(fit[,-1])
  }
  #Fit model for best lambda
  mad <- apply(ypred,2,function(z) mean(abs(z-y)))
  fit <- rq.lasso.fit(x[-omit,,drop=FALSE],y[-omit], method="lasso", tau = 0.5*(1+alpha), lambda=lambda[which.min(mad)])$coefficients
  return(c(0,fit[-1]))
}


lassoladcvfit <- function(y,x,k=10) {
    #Remove intercept, as slim cannot currently handle it
    n <- length(y)
    y <- y-mean(y)
    x <- x[,apply(x,2,'sd')>0] #remove intercept
    x <- scale(x,center=TRUE,scale=FALSE)
    #Define lambda values
    lambda.max = max(abs(crossprod(x, sign(y)/n)))
    lambda.min.value = sqrt(log(ncol(x))/n)/4
    lambda = exp(seq(log(lambda.max), log(lambda.min.value), length=10))
    #Run cross-validation
    all.folds <- split(sample(1:n), rep(1:k, length = n))
    ypred <- matrix(nrow=n,ncol=k)
    for (i in seq(k)) {
        omit <- all.folds[[i]]
        fit <- lassoladfit(x=x[-omit,,drop=FALSE], y=y[-omit], lambda=lambda)
        ypred[omit,] <- x[omit,] %*% fit
    }
    #Fit model for best lambda
    mad <- apply(ypred,2,function(z) mean(abs(z-y)))
    fit <- lassoladfit(x=x[-omit,,drop=FALSE], y=y[-omit], lambda=lambda[which.min(mad)])
    return(c(0,fit))
}

lassoladfit <- function(y,x,lambda) {
    #Fit LASSO-LAD for a series of lambda values, return coefficient estimates
    slim(X=x,Y=y,method="lq",q=1,lambda=lambda,verbose=FALSE)$beta
}


###########################################################################################
## 3. ROUTINES TO SUMMARIZE SIMULATION RESULTS
###########################################################################################

getHPM <- function(z) {
    #Extract highest posterior probability model
    # Input
    # - z: output returned by simfit
    # - theta: true value of regression coefficients
    pp= z$pp.n; modelid <- sub('1,','',as.character(pp$modelid)); bestmodel.n <- modelid[1]
    pp= z$pp.skewn; modelid <- sub('1,','',as.character(pp$modelid)); bestmodel.skewn <- modelid[1]
    pp= z$pp.l; modelid <- sub('1,','',as.character(pp$modelid)); bestmodel.l <- modelid[1]
    pp= z$pp.skewl; modelid <- sub('1,','',as.character(pp$modelid)); bestmodel.skewl <- modelid[1]
    pp= z$pp.auto; modelid <- sub('1,','',as.character(pp$modelid)); bestmodel.auto <- modelid[1]
    bestmodel= c(n=bestmodel.n,skewn=bestmodel.skewn,l=bestmodel.l,skewl=bestmodel.skewl,auto=bestmodel.auto)
    return(bestmodel)
}


getCorrect <- function(z,theta) {
    #Extract posterior probability of simulation truth, proportion of correct model selections, average number of false positives and true positives
    # Input
    # - z: output returned by simfit
    # - theta: true value of regression coefficients
    # Note: intercept term (assumed to be first element in theta) is not considered, as it has different interpretations under symmetric and asymmetric errors
    truesel <- theta!=0
    truemodelid <- as.character(which(truesel)); truemodelid <- truemodelid[truemodelid != '1']
    truemodel <- paste(truemodelid,collapse=','); truemodel <- sub('^1,','',truemodel)
    #Assumed Normal residuals
    selmodel.n <- do.call(rbind, lapply(sims.out, function(z) z$pp.n[1,]))
    selmodel.n$modelid <- sub('^1,','',as.character(selmodel.n$modelid))  #discard intercept
    correctmodel <- mean(selmodel.n$modelid==truemodel)
    pptruemodel.n <- mean(sapply(sims.out, function(z) { sel <- which(sub('^1,','',as.character(z$pp.n$modelid))==truemodel); ifelse(length(sel)>0, sum(z$pp.n$pp[sel]), 0) } ))
    modelid <- strsplit(selmodel.n$modelid, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel[-1],truesel[-1]))) } ))
    ans.n= c(pptruemodel.n=pptruemodel.n,correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #Assumed Two-piece normal residuals
    selmodel.skewn <- do.call(rbind, lapply(sims.out, function(z) z$pp.skewn[1,]))
    selmodel.skewn$modelid <- sub('^1,','',as.character(selmodel.skewn$modelid))  #discard intercept
    correctmodel <- mean(selmodel.skewn$modelid==truemodel)
    pptruemodel.skewn <- mean(sapply(sims.out, function(z) { sel <- which(sub('^1,','',as.character(z$pp.skewn$modelid))==truemodel); ifelse(length(sel)>0, sum(z$pp.skewn$pp[sel]), 0) } ))
    modelid <- strsplit(selmodel.skewn$modelid, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel[-1],truesel[-1]))) } ))
    ans.skewn= c(pptruemodel.skewn=pptruemodel.skewn,correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #Assumed Laplace residuals
    selmodel.l <- do.call(rbind, lapply(sims.out, function(z) z$pp.l[1,]))
    selmodel.l$modelid <- sub('^1,','',as.character(selmodel.l$modelid))  #discard intercept
    correctmodel <- mean(selmodel.l$modelid==truemodel)
    pptruemodel.l <- mean(sapply(sims.out, function(z) { sel <- which(sub('^1,','',as.character(z$pp.l$modelid))==truemodel); ifelse(length(sel)>0, sum(z$pp.l$pp[sel]), 0) } ))
    modelid <- strsplit(selmodel.l$modelid, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel[-1],truesel[-1]))) } ))
    ans.l= c(pptruemodel.l=pptruemodel.l,correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #Assumed Two-piece Laplace residuals
    selmodel.skewl <- do.call(rbind, lapply(sims.out, function(z) z$pp.skewl[1,]))
    selmodel.skewl$modelid <- sub('^1,','',as.character(selmodel.skewl$modelid))  #discard intercept
    correctmodel <- mean(selmodel.skewl$modelid==truemodel)
    pptruemodel.skewl <- mean(sapply(sims.out, function(z) { sel <- which(sub('^1,','',as.character(z$pp.skewl$modelid))==truemodel); ifelse(length(sel)>0, sum(z$pp.skewl$pp[sel]), 0) } ))
    modelid <- strsplit(selmodel.skewl$modelid, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel[-1],truesel[-1]))) } ))
    ans.skewl= c(pptruemodel.skewl=pptruemodel.skewl,correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #Inferred residuals
    selmodel.auto <- do.call(rbind, lapply(sims.out, function(z) z$pp.auto[1,]))
    selmodel.auto$modelid <- sub('^1,','',as.character(selmodel.auto$modelid))  #discard intercept
    correctmodel <- mean(selmodel.auto$modelid==truemodel)
    pptruemodel.auto <- mean(sapply(sims.out, function(z) { sel <- which(sub('^1,','',as.character(z$pp.auto$modelid))==truemodel); ifelse(length(sel)>0, sum(z$pp.auto$pp[sel]), 0) } ))
    modelid <- strsplit(selmodel.auto$modelid, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel[-1],truesel[-1]))) } ))
    ans.auto= c(pptruemodel.auto=pptruemodel.auto,correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    ans <- data.frame(method=c('Normal','Two-piece Normal','Laplace','Two-piece Laplace','Inferred'),rbind(ans.n,ans.skewn,ans.l,ans.skewl,ans.auto))
    return(ans)
}

getCorrect.pen <- function(sims.pen,theta=theta) {
    #Compute proportion of correct model selections, average number of false positives and true positives
    # - sims.pen: output from simpen.lhood
    # - theta: true values of regression coefficients
    # Note: intercept (assumed to be first element in theta) is not considered for consistency with function getCorrect
    truemodelid <- as.character(which(theta!=0))
    truemodel <- paste(truemodelid,collapse=',')
    #LASSO-QR
    sel.qr <- sapply(sims.pen, function(z) z$coef$lasso.qr.coef!=0)
    sel.qr <- apply(sel.qr, 2, function(z) paste(which(z),collapse=','))
    correctmodel <- mean(truemodel == sel.qr)
    modelid <- strsplit(sel.qr, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel,theta!=0))) } ))
    ans.qr <- c(correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #LASSO-LS
    sel.ls <- sapply(sims.pen, function(z) z$coef$lasso.ls.coef!=0)
    sel.ls <- apply(sel.ls, 2, function(z) paste(which(z),collapse=','))
    correctmodel <- mean(truemodel == sel.ls)
    modelid <- strsplit(sel.ls, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel,theta!=0))) } ))
    ans.ls <- c(correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #LASSO-LAD
    sel.qr0 <- sapply(sims.pen, function(z) z$coef$lasso.qr0.coef!=0)
    sel.qr0 <- apply(sel.qr0, 2, function(z) paste(which(z),collapse=','))
    correctmodel <- mean(truemodel == sel.qr0)
    modelid <- strsplit(sel.qr0, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel,theta!=0))) } ))
    ans.lad <- c(correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    #SCAD
    sel.scad <- sapply(sims.pen, function(z) z$coef$scad.coef!=0)
    sel.scad <- apply(sel.scad, 2, function(z) paste(which(z),collapse=','))
    correctmodel <- mean(truemodel == sel.scad)
    modelid <- strsplit(sel.scad, split=',')
    correctvars <- mean(sapply(modelid, function(z) { varsel= rep(FALSE,length(theta)); varsel[as.numeric(z)]= TRUE; sum(diag(table(varsel,theta!=0))) } ))
    ans.scad <- c(correctmodel=correctmodel,correctvars=correctvars,rowMeans(sapply(modelid, function(z) c(fp=sum(!(z %in% truemodelid)), tp= sum(z %in% truemodelid)))))
    ans <- data.frame(method=c('LASSO-LS','LASSO-LAD','LASSO-QR','SCAD'),rbind(ans.ls,ans.lad,ans.qr,ans.scad))
    return(ans)
}
